/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.tags;

import static org.apache.commons.lang.StringEscapeUtils.escapeXml;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.jstl.fmt.LocalizationContext;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.apache.taglibs.standard.tag.common.fmt.BundleSupport;

import com.mysema.commons.fluxml.XMLWriter;
import com.mysema.commons.web.ContextURL;

/**
 * a_tag provides a convenience shortcut for HTML a tag, JSTL c:url and JSTL fmt:message
 *
 * @author tiwe
 * @version $Id$
 */
public class a_tag extends SimpleTagSupport implements DynamicAttributes{
    
    private String href, key, title, label;
    
    private boolean absolute = false, secure = false, encode = true;
    
    private Map<String,String> attributes = new HashMap<String,String>();  

    public void setDynamicAttribute(String uri, String localName, Object value)
            throws JspException {
        attributes.put(localName, escapeXml(value.toString()));        
    }
    
    public void doTag() throws JspException, java.io.IOException {
        PageContext jspContext = (PageContext) getJspContext();
        HttpServletRequest request = (HttpServletRequest) jspContext.getRequest();
        HttpServletResponse response = (HttpServletResponse) jspContext.getResponse();

        String link = href;        
        if (encode){
            link = ContextURL.resolve(request, response, href, absolute, secure);
        }
                
        String bodyContent = null;
        if (key != null || title != null || label != null){
            LocalizationContext context = BundleSupport.getLocalizationContext(jspContext);
            String bundleKey = key;
            if (title != null)
                bundleKey = title + ".title";
            else if (label != null)
                bundleKey = label + ".label";
            
            bodyContent = context.getResourceBundle().getString(bundleKey);
            
        }else{
            StringWriter body = new StringWriter();        
            getJspBody().invoke(body);
            bodyContent = body.toString();
        }
        
        createLinkMarkup(jspContext.getOut(), request, href, link, bodyContent);
        
    }
        
    protected void createLinkMarkup(JspWriter out, HttpServletRequest request, String href, String link, String bodyContent) throws IOException {
        XMLWriter xml = new XMLWriter(out);
        xml.begin("a").attribute("href", link);       
        xml.attributes(attributes);
        xml.printRaw(bodyContent).end("a");
    }

    public void setAbsolute(Boolean absolute) {
        this.absolute = absolute != null ? absolute.booleanValue() : false;
    }

    public void setSecure(Boolean secure) {
        this.secure = secure != null ? secure.booleanValue() : false;
    }

    public void setHref(String href) {
        this.href = href;
    }
    
    public void setEncode(Boolean encode) {
        this.encode = encode != null ? encode.booleanValue() : false;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setLabel(String label) {
        this.label = label;
    }
    
    
    
}
